@@ -5,7 +5,6 @@ import org.json.simple.JSONObject;
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 import java.io.UnsupportedEncodingException;
-import java.net.URLDecoder;
 import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -118,15 +117,22 @@ public class JmxRequest extends JSONObject {
     private String operation;
     private Type type;
 
+    // Max depth of returned JSON structure when deserializing.
+    private int maxDepth = 0;
+    private int maxCollectionSize = 0;
+    private int maxObjects = 0;
+
     private static final Pattern SLASH_ESCAPE_PATTERN = Pattern.compile("^-*\\+?$");
 
-    JmxRequest(String pPathInfo) {
+    JmxRequest(String pPathInfo, Map pParameterMap) {
         try {
             if (pPathInfo != null && pPathInfo.length() > 0) {
 
                 // Get all path elements as a reverse stack
                 Stack<String> elements = extractElementsFromPath(pPathInfo);
-
+                if (elements.size() == 0) {
+                    throw new IllegalArgumentException("No request type given");
+                }
                 type = extractType(elements.pop());
 
                 Processor processor = processorMap.get(type);
@@ -146,6 +152,17 @@ public class JmxRequest extends JSONObject {
                 put("type",type.getValue());
                 processor.setupJSON(this);
             }
+            if (pParameterMap != null) {
+                if (pParameterMap.get("maxDepth") != null) {
+                    maxDepth = Integer.parseInt( ((String []) pParameterMap.get("maxDepth"))[0]);
+                }
+                if (pParameterMap.get("maxCollectionSize") != null) {
+                    maxCollectionSize = Integer.parseInt(((String []) pParameterMap.get("maxCollectionSize"))[0]);
+                }
+                if (pParameterMap.get("maxObjects") != null) {
+                    maxObjects = Integer.parseInt(((String []) pParameterMap.get("maxObjects"))[0]);
+                }
+            }
         } catch (NoSuchElementException exp) {
             throw new IllegalArgumentException("Invalid path info " + pPathInfo,exp);
         } catch (MalformedObjectNameException e) {
@@ -154,6 +171,8 @@ public class JmxRequest extends JSONObject {
                             "\": " + e.getMessage(),e);
         } catch (UnsupportedEncodingException e) {
             throw new IllegalStateException("Internal: Illegal encoding for URL conversion: " + e,e);
+        } catch (EmptyStackException exp) {
+            throw new IllegalArgumentException("Invalid arguments in pathinfo " + pPathInfo + " for command " + type,exp);
         }
     }
 
@@ -272,9 +291,6 @@ public class JmxRequest extends JSONObject {
         }
     }
 
-
-
-
     public String getValue() {
         return value;
     }
@@ -287,6 +303,17 @@ public class JmxRequest extends JSONObject {
         return operation;
     }
 
+    public int getMaxDepth() {
+        return maxDepth;
+    }
+
+    public int getMaxCollectionSize() {
+        return maxCollectionSize;
+    }
+
+    public int getMaxObjects() {
+        return maxObjects;
+    }
 
     @Override
     public String toString() {
@@ -398,5 +425,6 @@ public class JmxRequest extends JSONObject {
                 r.put("mbean",r.objectName.getCanonicalName());
             }
         });
+
     }
 }
